#define  FT                   700.0     //     . ".0"   - !
#define  COEFF_DASH           3.2       //      

#include "common_defs.h"
#include "SineTable.h"
#include "Morse.h"

/*************************************************************************
 *                                                   *
 *************************************************************************/
UINT16 volatile InterruptsCounter = 0;
UINT16 volatile CurrentDelay = 0;
UINT16 volatile CurrentSineValue = 0;
UINT16 volatile Amplitude = 0;

UINT8 volatile  CurrentPlayingSign = 0xFF; //   . DASH - , DOT - , FF -   
UINT8 volatile  play_flag = 0xFF;          // -       . DASH - , DOT - , FF -   

UINT16 dot_length;      //     

// ,      "CQ DE CALL"
//       S3
const UINT8 PROGMEM text_call1[] = "CQ DE RK0LXA\0"; //  1
const UINT8 PROGMEM text_call2[] = "CQ DE RC0LKT\0"; //  2
const UINT8 PROGMEM text_call3[] = "CQ DE RZ0LWA\0"; //  3

/*************************************************************************
 *                                                         *
 *************************************************************************/
int main(void)
{
   UINT8 tmp_play_flag;

   Init();    // 
   ADCInit(); //  ADC

   //  'A'  
   PlaySign(DOT);
   PlaySign(DASH);

   for (;;)
   {
      if (0 == (PIND & _BV(PD2)))      // PD2 -  
         play_flag = DOT;  //   
      else if (0 == (PIND & _BV(PD3))) // PD3 -  
         play_flag = DASH; //   
      else //     -  (      )    INT0, INT1  INT2
      {
         GIFR  |= (_BV(INTF0)|_BV(INTF1)|_BV(INTF2)); //     -   ,   
         GICR  |= (_BV(INT0)|_BV(INT1)|_BV(INT2));    //    INT0,INT1,INT2
         sleep_mode();                 //        ,     
         GICR  &= ~(_BV(INT0)|_BV(INT1)|_BV(INT2));   //    INT0,INT1,INT2
      }

      //       ,   "play_flag"    (0xFF)
      // ,     ""  1 
      while (0xFF != play_flag)
      {
         tmp_play_flag = play_flag;
         play_flag = 0xFF;
         PlaySign(tmp_play_flag); //     "play_flag"  
      }
   }
   return 0;
}

/**************************************************************************
 *                                                       *
 **************************************************************************/
void PlaySign(UINT8 sign)
{
   UINT8  tmpL;
   UINT8  tmpH;
   UINT16 delay;

   CurrentPlayingSign = sign;

   ADMUX |= _BV(REFS0) | _BV(MUX2) | _BV(MUX1); //    AVCC   ADC6 (34  MCU)
   wait_us(265);                                //      260 
   ADCwork();                                   //   
   wait_us(265);                                //      260 
   ADCwork();                                   //     
   tmpL = ADCL;
   tmpH = ADCH;
   dot_length = (((tmpH << 8) + tmpL) * 1.8) + 340; // - 1.8   ,   340 
                                                    //      ,  340 - 
                                                    //  "" 

   PressPTT();

   if (sign)
      delay = dot_length;
   else
      delay = COEFF_DASH * dot_length;
   Wait(delay);             /*      */

   ReleasePTT();

   Wait(dot_length);        /*     */

   CurrentPlayingSign = 0xFF;
}

/**************************************************************************
 *                                                                   *
 **************************************************************************/
void Wait(UINT16 delay)
{
   TIMSK  |= _BV(TOIE1);             /*     T1 */

   CurrentDelay = delay;
   Amplitude = 0;                    /*     0-  */
   CurrentSineValue = 0;

   for (;;)
   {
      sleep_mode();                  /* ,     */
      /*      ,   play_flag =  */
      if ((DOT == CurrentPlayingSign) && (0 == (PIND & _BV(PD3)))) play_flag = DASH; // PD3 -  
      /*      ,   play_flag =  */
      if ((DASH == CurrentPlayingSign) && (0 == (PIND & _BV(PD2)))) play_flag = DOT; // PD2 -  

      if (InterruptsCounter > delay)
      {
         InterruptsCounter = 0;
         break;
      }
   }

   TIMSK  &= ~_BV(TOIE1);           /*     T1 */
}

/***************************************************************************
 *     - 1                *
 *  PWM-,    Timer/Counter1    OCR1A *
 * ,    (SREG I-bit = 1 and TOIE1 = 1)     *
 *    TIMER1_OVF_vect.                             *
 ***************************************************************************/
ISR(TIMER1_OVF_vect)
{
   InterruptsCounter++;
   
   if (InterruptsCounter < 127)
   {
      if (Amplitude < 127) Amplitude++;            //     
   }
   else if (InterruptsCounter > (CurrentDelay - 126))
   {
      if (Amplitude > 1) Amplitude--;              //     
   }

   CurrentSineValue += MORSE_FREQUENCY_STEP;       //     

   OCR1A = (__LPM(&SmoothingTable[Amplitude]) * __LPM(&SineTable[(UINT8)(((CurrentSineValue+4) >> 3)&(0x007F))])) >> 8; //   
}

/*****************************
 *        *
 *****************************/
void wait_ms(UINT16 pause)
{
   UINT16 i;
   for (i = 0; i < pause; i++)
   {
      wait_us(1000);
   }
}

/*****************************
 *        *
 *****************************/
void wait_us(UINT16 pause)
{
   UINT16 i;
   for (i = 0; i < pause; i++)
   {
      nop();
      nop();
   }
}

/****************************************************************************
 *                                             *
 ****************************************************************************/
ISR(TIMER2_COMP_vect)
{}
ISR(TIMER2_OVF_vect)
{}
ISR(TIMER1_CAPT_vect)
{}
ISR(TIMER1_COMPA_vect)
{}
ISR(TIMER1_COMPB_vect)
{}
ISR(TIMER0_OVF_vect)
{}
ISR(SPI_STC_vect)
{}
ISR(USART_RXC_vect)
{}
ISR(USART_UDRE_vect)
{}
ISR(USART_TXC_vect)
{}
ISR(ADC_vect)
{}
ISR(EE_RDY_vect)
{}
ISR(ANA_COMP_vect)
{}
ISR(TWI_vect)
{}
ISR(TIMER0_COMP_vect)
{}
ISR(SPM_RDY_vect)
{}
ISR(INT0_vect)
{}
ISR(INT1_vect)
{}

/**************************************************************************
 *     INT2 -   "CALL"  "CQ DE CALL"  *
 **************************************************************************/
ISR(INT2_vect)
{
   UINT16 letter;
   UINT8  symbol_code;
   UINT8  symbol_length;
   UINT8  i;
   UINT8  call_only_flag;                     //    ,  CQ DE  
   const  UINT8 *p_text;

   GICR  &= ~(_BV(INT0)|_BV(INT1)|_BV(INT2)); //    INT0,INT1,INT2
   sei();
   //   ,     CALL.  8 , 80   
   call_only_flag = 0;
   for (i=0; i<8; i++)
   {
      if (PIND & _BV(PD6)) call_only_flag++;
      wait_ms(10);
   }

   //          S3
   for (;;)
   {
      switch (PIND & (_BV(PD0)|_BV(PD1)))
      {
        case 0:
           p_text = &text_call1[0];           // RC0LKT
           break;
        case 1:
           p_text = &text_call2[0];           // RK0LXA
           break;
        case 2:
           p_text = &text_call3[0];           // RZ0LWA
           break;
        default:
           p_text = &text_call1[0];
      }

      if (call_only_flag) p_text += 6;        //    6  ,   "CQ DE "

      for(;;)
      {
         letter = (UINT16)__LPM(p_text++);
         if (!letter) break;
         
         if (SPACE != letter)
         {
            letter        = (letter - EXCLAMATION_MARK)<<1;
            symbol_code   = __LPM((UINT16)(&morse_lookup_table[0]+letter++));   //p_morse_lookup_table[letter].symbol_code;
            symbol_length = __LPM((UINT16)(&morse_lookup_table[0]+letter));     //p_morse_lookup_table[letter].symbol_length;
   
            //   
            for (; symbol_length > 0; --symbol_length)
            {
               if (symbol_code & 0x80)             //      
                  PlaySign(DOT);
               else
                  PlaySign(DASH);
   
               symbol_code <<= 1;
            }
            Wait((COEFF_DASH * dot_length) - dot_length); //   
         }
         else
         {
            Wait(6 * dot_length);      //     
         }
      }
      if (!(PINB & _BV(PB2))) break;   //   ,    
      Wait(10 * dot_length);           //   
   }

   GIFR  |= (_BV(INTF0)|_BV(INTF1)|_BV(INTF2)); //     -   ,   
   GICR  |= (_BV(INT0)|_BV(INT1)|_BV(INT2));    //    INT0,INT1,INT2
   play_flag = 0xFF;  //  ,                  
}


/************************************************************************
 *                                                      *
 ************************************************************************/
void PressPTT(void)
{
   PORTC   = (PORTC & ~_BV(PC2)) | _BV(PC3); //PC2 = 0, PC3 = 1
   TCNT1   = 0;                 /*  ,      ,     */
   TCCR1A |= _BV(COM1A1);       /*  - 1   OC1 */
}

/*************************************************************************
 *                                                      *
 *************************************************************************/
void ReleasePTT(void)
{
   PORTC  = (PORTC & ~_BV(PC3)) | _BV(PC2); //PC2 = 1, PC3 = 0
   TCCR1A &= ~_BV(COM1A1);     //  - 1   OC1
   PORTD  &= ~_BV(PD5);        //  -   0   
}

/************************************************************************
 * +-----+-----+--------------------------------------------------------+
 * ISC11ISC10                                                
 * +-----+-----+--------------------------------------------------------+
 * 0    0      .-  .   INT1
 * +-----+-----+--------------------------------------------------------+
 * 0    1                          
 * +-----+-----+--------------------------------------------------------+
 * 1    0            INT1 
 * +-----+-----+--------------------------------------------------------+
 * 1    1        .    INT1 
 * +-----+-----+--------------------------------------------------------+
 *************************************************************************/
void Init(void)
{
   cli();                          /*      !!!
                                       -   ... */
   ACSR    |= _BV(ACD);            /*      */
   wdt_disable();                  /*  WDT */
   MCUCSR  |= _BV(JTD);            /*  JTAG (         ) */

   PORTA   = 0x00;                 /*    */
   DDRA    = ~_BV(PA6);            /*  PA6(ADC6)  ,  -  */

   PORTB   = _BV(PB2);             /*    PB2 (INT2) */
   DDRB    = ~_BV(PB2);            /*  PB2(INT2)  ,  -  */

   PORTC   = 0x00;                 /*    */
   DDRC    = 0xFF;                 /*   , ,     PC2  PC3 (/TX  TX) */

   PORTD  |= (_BV(PD0)|_BV(PD1)|_BV(PD2)|_BV(PD3));    /*    PD0,PD1,PD2,PD3 (  14,15,16-INT0,17-INT1) */
   DDRD    = 0xB0;                 /*  PD0,PD1,PD2,PD3  PD6 (  14,15,16-INT0,17-INT1, 20)  ,  - , ,     PD5 (OC1A) */

   TCCR1B  = _BV(CS10);            /* CLK/1 + TCNT1 Start*/
   TCCR1A  = _BV(WGM10);           /* 8-Bit phase-correct PWM */

   wait_ms(100);                   /*    ,        */

   MCUCSR |= _BV(ISC2);                                      /*  INT2      PB2
                                                                (   , !) */

   MCUCR  |= (_BV(ISC11)|_BV(ISC01));                        /*      INT0,INT1 */

   GICR   |= _BV(INT0)|_BV(INT1)|_BV(INT2);                  /*    INT0,INT1,INT2 */
   
   sei();
}
